c
c
c
c
c      If you are looking at this file from an internet browser,
c      then choose "save as" from your browser and put a copy
c      on your computer with the name "elas.f". See below for 
c      setup instructions.
c
c      A C++ verson of this program is available upon request, 
c      but it is not quite as bulletproof.
c
c
c
c
c
c
c
c
c
************************************************************************
*                                                                      *
*  copyright 1999, 2007 Rebecca Brannon (brannon@mech.utah.edu)        *
*  All rights reserved.                                                *
*                                                                      *
*  To our knowledge, this program is safe. It has never been reported  *
*  to have any bugs or unexpected behavior. Nevertheless, be advised:  *
*  Compiling or executing this program constitutes your agreement to   *
*  assume responsibility for any damages directly or indirectly        *
*  associated with this program.                                       *
*  Use this program at your own risk!                                  *
*                                                                      *
*  Compile this program by typing (from a UNIX command tool)           *
*                                                                      *
*      f77 -o elas elas.f                                              *
*                                                                      *
*  Run the program by typing "elas"                                    *
*  If typing "elas" doesn't work, try "./elas" (and ask your computer  *
*  support person to explain how to modify your path so that you can   *
*  simply type "elas" from any directory).                             *
*                                                                      *
************************************************************************
c
c
c
c
!
!       W h a t   d o e s   t h i s    p r o g r a m    d o?
!
!
!Given any two INDEPENDENT elastic constants (from the list below)
!this program will compute all of the others. This program also contains
!formulas that will give you an estimate of how the elastic properties
!will change when the material contains pores or cracks. If you don't
!care about this information, simply enter zero (0) when asked to 
!provide a damage measure (or just hit return because undamaged
!properties are the default). If you ARE interested in getting the
!changes in elastic properties that result from PORES, then first 
!enter any two independent elastic properties of the nonporous matrix
!material, and then enter the pore volume fraction when asked for the 
!damage measure. If you seek the changes in elastic properties resulting
!from CRACKS, then specify the NEGATIVE of the number of cracks per
!unit volume times the mean of the cube of the crack radius (using
!a negative is simply a low-tech way of telling this program that
!the damage measure represents crack data instead of porosity data).
!Again note: the first two parameters must be the moduli of the
!undamaged matrix material. Solving the inverse problem of getting
!the undamaged moduli when the damaged moduli are known requires an
!iterative solver -- contact me (brannon@mech.utah.edu) if you need 
!routines that will do that.
!
!
!
!    Specify your known input like this:
!
!       FIRST elastic constant?
!      E=200e9                   <--- MATRIX VALUE IF CRACKED/POROUS
!       SECOND elastic constant?
!      G=79e9                    <--- MATRIX VALUE IF CRACKED/POROUS
!       Density? [0] (enter zero if wave speeds not desired)
!      7860
!       Damage measure? [0]
!      .04
!
!Defaults [in brackets] may be accepted by hitting return. After 
!entering data one time, the program will loop back for a new set of 
!inputs, and all your old inputs will be the new defaults (very useful 
!for correcting typos).
! 
!  A NOTE ABOUT WAVE SPEEDS...
!  When shock physicists say `cs', they mean what we call `cl'.
!  Longitudinal waves (called "P-waves" by geologists) travel at
!  speed `cl'.  Shear waves ("S-waves") travel at speed ct.
!  When you compute elastic moduli using wave speeds, be aware that
!  you are actually finding the ISENTROPIC moduli. Conversely, if
!  you seek the wave speeds, then you must enter ISENTROPIC moduli.
!  For isotropic materials, converting between isentropic moduli
!  and isothermal moduli is straightforward, but this program 
!  won't do it for you.
!  Wave speeds correspond to ISENTROPIC moduli because sound waves
!  travel such that the stress-strain response is the same as that
!  when the entropy is constant. Elastic moduli that are measured
!  quasistatically using a load cell are typically determined from
!  stress-strain response at constant temperature (e.g., room temperature)
!  Here is how the conversions are computed... Let subscripts "s" 
!  and "T" indicate isentropic and isothermal, respectively. Then
!  relevant formulas are: G_s = G_T and K_s/K_T = c_p/c_v,
!  where c_p and c_v are specific heats at const. pressure and volume.
!  If you don't have values for both types of specific heats, consider
!  looking at http://mech.utah.edu/~brannon/DerivativeRecursionTables.pdf
!  for alternative formulas.
!
!
!  #############################################################
!  lam = First Lame parameter         = G(E-2G)/(3G-E) = K-2G/3
!    G = Shear modulus (= 2nd Lame parameter, mu)   = E/2/(1+nu)
!    E = Young's modulus              = 3K(1-2nu) = 2G(1+nu) = 9KG/(3K+G)
!   nu = Poisson's ratio              = (3K-E)/6K = lam/2/(lam+G)
!    K = bulk modulus                 = E/3/(1-2nu) = lam + 2G/3
!    H = constrained modulus          = 2G+lam = 3K-2lam = K + 4G/3
!   ko = SIGy/SIGx in uniaxial strain = nu/(1-nu)
!   cl = longitudinal wave speed      = sqrt(H/rho)
!   ct = shear (TRANSVERSE) wave speed  = sqrt(G/rho)
!   co = bulk/plastic wave speed      = sqrt(K/rho)=SQRT(cl^2-4(ct^2)/3)
!   cr = thin rod elastic wave speed  = sqrt(E/rho)
!  #############################################################
!
!To see this help info again, type "help" at any prompt.
!Type  q or e to stop.
c---.----1----.----2----.----3----.----4----.----5----.----6----.----7--

      program elastic_constants
      implicit none
      character jnkstr*300,JNKchr*1,option*2,argv*60
      character jstrng*300

c     The HELPFILE is simply the prolog of this FORTRAN source code file
c     (i.e., the set of lines above that begin with "!"). If you wish for
c     that information to be available by typing "help", then modify
c     the two lines below to point to the full path the elas.f file.
c     If the code is unable to locate that file, then it will print out
c     much shorter alternative help information.
c     In other words, editing the code is optional.
      character HELPFILE*49
c      ----.----1----.----2----.----3----.----4----.----5----.----6----.
      parameter (HELPFILE=
     $'/home/rmbrann/Fort/Apps/StandAlonePrograms/elas.f')


      integer LNBL77,index,iargc,number_of_arguments,ierr
      integer mLstring,ncnst,idum
      parameter (mLstring=50,ncnst=11)
      integer i,j,ieq,J1,J2,icase,ilast
      real*8
     &     r0,r1,r2,r3,r4,r5,r6,r8,r9,r10,r15,puny,r32,r45,r16
      parameter (r0=0.0d0,r1=0.1d1,r2=0.2d1,r3=0.3d1,r4=0.4d1,
     &           r5=0.5d1,r6=0.6d1,r8=0.8d1,r9=0.9d1,r10=0.1d2,
     &           r15=0.15d2,puny=0.1d-99,r32=0.32d2,r45=0.45d2,
     &           r16=0.16d2)
c     ---------------------------------------------------------------
c     The following are tied to the NAMcnst array. If that array
c     is changed, then these should be changed too. Also update
c     these pointers in chkValidity subroutine and fix coding that
c     uses "icase".
      integer iplam,ipG,ipE,ipnu,ipK,ipH,ipko,ipcl,ipct,ipco,ipcr
      integer ipneedrho
      parameter (iplam=1)
      parameter (ipG  =2)
      parameter (ipE  =3)
      parameter (ipnu =4)
      parameter (ipK  =5)
      parameter (ipH  =6)
      parameter (ipko =7)
      parameter (ipneedrho=8)
      parameter (ipcl =8)
      parameter (ipct =9)
      parameter (ipco =10)
      parameter (ipcr =11)
c     ---------------------------------------------------------------
      integer nprompt,iprompt
      parameter (nprompt=2)
      character*3 nam(nprompt)
      integer idv(nprompt)
      integer Lpstring(nprompt),LstringRHO,LstringPORO
      real*8
     &        lam,G,E,nu,K,H, cnst(ncnst),val(nprompt),rho,poro,A,B,
     &       om2n,omn,opn,r2g  ,RNU, ko,fm,alpha,beta,poroDEFAULT
      character*3 NAMcnst(ncnst)
      character*(mLstring) string,pstring(nprompt),stringRHO,stringPORO
      character bang*1
      character*30 prompt(nprompt)
      parameter (bang = '!')
      logical syntax,syntaxRHO, syntaxPORO
      dimension syntax(nprompt)
      logical valid(nprompt),validRHO,
     &        validPORO
      logical verbose
      logical single_line_output
      logical print_tensor
      logical damage
      logical debug
      data LstringRHO/1/
      data NAMcnst/'lam','G','E','nu','K','H','ko','cl','ct','co','cr'/
      data stringRHO/'0'/,rho/r0/
      data fm/r0/
      data stringPORO/'0'/,poro/r0/,poroDEFAULT/r0/
      data prompt/ 'FIRST elastic constant?',
     $             'SECOND elastic constant?'/
      data syntaxRHO/.true./, syntaxPORO/.true./
      data validRHO/.true./,   validPORO/.false./
      data verbose/.false./
      data single_line_output/.false./
      data print_tensor/.false./
      data damage/.false./
      data debug/.false./
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
          do iprompt=1,nprompt
             syntax(iprompt)=.false.
             valid(iprompt)=.false.
             pstring(iprompt)='pstring-not-yet-defined'
             nam(iprompt)='nam-not-yet-defined'
             val(iprompt)=r0
             Lpstring(iprompt)=0
             idv(iprompt)=0
          enddo
          do i=1,ncnst
             cnst(i)=r0
          enddo
c
c         The following bit of programming works for some platforms
c         that support command line arguments. This program is platform
c         independent when run WITHOUT any command line arguments.
c
          number_of_arguments = iargc()
          do i=1,number_of_arguments
            call getarg(i,argv)

            IF (argv(1:1).eq.'-') THEN
c
c             THE ARGUMENT IS AN OPTION
c
              idum=lnbl77(argv)  
              do j=2,idum
                option = argv(j:j)
                if (option.eq.'a')then
                  print*,'RUNNING IN APREPRO OUTPUT MODE'
                  single_line_output = .true.
                elseif (option.eq.'d') then
                  debug = .true.
                  verbose = .true.
                  print*,'RUNNING IN VERBOSE+DEBUG MODE'
                elseif (option.eq.'v')then
                  verbose = .true.
                  print*,'RUNNING IN VERBOSE MODE'
                elseif (option.eq.'s')then
                  single_line_output = .true.
                elseif (option.eq.'t')then
                  print_tensor=.true.
                end if
              enddo
            END IF
          end do


c     Remind user how this program works...
      
      call prtHELP(HELPFILE,ierr)
      print*


c     Begin infinite loop
   71 continue

      iprompt=1
   72 continue
      if(iprompt.gt.2)go to 73
      if(verbose)print*,'iprompt=',iprompt
      if(verbose)call prna(cnst,NAMcnst,ncnst,'cnst array')
      if(syntax(iprompt).and.valid(iprompt))then
        print*,prompt(iprompt),' [',
     $         pstring(iprompt)(1:Lpstring(iprompt)),']'
      else
        print*,prompt(iprompt)
      end if
      read(*,1)string
      idum=index(string,bang)
      if (idum.ne.0) string=string(1:idum-1)
      if(LNBL77(string).eq.0)then
         if(syntax(iprompt).and.valid(iprompt))then
            iprompt=iprompt+1
            go to 72
         else if(.not.syntax(iprompt))then
            go to 992
         else
            print*,'    ***********************************'
            print*,'    There is no currently valid default'
            print*,'           t r y    a g a i n'
            print*,'    ***********************************'
            go to 72
         end if
      end if
      idum=len(string)
      call compress(string,idum)
      ieq=index(string,'=')
      IF(ieq.le.0)THEN
        JNKchr=string(1:1)
        call ChngCase('L',JNKchr)
        IF(string.eq.'q' .or. string.eq.'e')THEN
          stop
        ELSEIF(JNKchr.eq.'v')THEN
          print*,' '
          print*,' '
          print*,' '
          print*,'   !!!!!!!!!!!!!!!!!!!!'
          print*,'   verbose mode enabled'
          print*,'   (disable with "nov")'
          print*,'   !!!!!!!!!!!!!!!!!!!!'
          print*,' '
          print*,' '
          verbose= .true.
          go to 72
        ELSEIF(string(1:3).eq.'nov')THEN
          print*,'   verbose mode disabled'
          verbose=.false.
          go to 72
        ELSEIF(JNKchr.eq.'d')THEN
          print*,' '
          print*,' '
          print*,' '
          print*,'   !!!!!!!!!!!!!!!!!!!!'
          print*,'   debug mode enabled'
          print*,'   (disable with "nod")'
          print*,'   !!!!!!!!!!!!!!!!!!!!'
          print*,' '
          print*,' '
          verbose= .true.
          debug = .true.
          go to 72
        ELSEIF(string(1:3).eq.'nod')THEN
          print*,'   debug mode disabled'
          print*,'   verbose mode disabled'
           debug=.false.
           verbose=.false.
          go to 72
        ELSE IF(JNKchr.eq.'h')THEN
          call prtHELP(HELPFILE,ierr)
          print*,'------------------------'
          print*,'Optional commands:'
          print*,'(accepted at any prompt or as command line option):'
          print*,'a: turn on aprepro-style output'
          print*,'d: turn on debug+verbose mode'
          print*,'s: single line (silent) output'
          print*,'t: turn out tensor-style output'
          print*,'v: turn on verbose mode to see more info and results'
          print*,'------------------------'
          go to 992
        ELSE IF(JNKchr.eq.'s')then
          print*,' '
          print*,' '
          print*,' '
          print*,'   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
          print*,'   single line output mode enabled'
          print*,'   (disable with "nos")'
          print*,'   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
          print*,' '
          print*,' '
          single_line_output= .true.
          go to 72
        ELSEIF(string(1:3).eq.'nos')THEN
          print*,'   single-line output mode disabled'
           single_line_output=.false.
          go to 72
        ELSE IF(JNKchr.eq.'a')THEN
          single_line_output=.true.
           print*,' '
           print*,' '
           print*,' '
           print*,'   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
           print*,'   aprepro formats enabled.'
           print*,'   (disable with "noa")'
           print*,'   !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!'
           print*,' '
           print*,' '
          go to 72
        ELSE IF(string(1:3).eq.'noa')THEN
          single_line_output=.false.
          print*, 'aprepro formats disabled.'
          go to 72
        ELSE IF(JNKchr.eq.'t')THEN
          print_tensor=.true.
          print*,'-----------------------------------'
          print*, 'tensor formats enabled.'
          print*
          print*,'Our index mapping: 11,22,33,23,31,12'
          print*,'Connection with four-index components is'
          print*,'E11=E1111, E12=E1122, etc.'
          print*,'C11=C1111, C12=C1122, etc.'
          print*,'However, we follow Mandel two-index definitions'
          print*,'so that'
          print*
          print*,'E44 equals 2*E2323 (twice as big)'
          print*,'C44 equals 2*C2323 (twice as big)'
          print*,'and similarly for the other components with'
          print*,'indices greater than 3.'
          print*
          print*,'Consequently, be aware that'
          print*
          print*,'E2323=E3131=E1212 = G'
          print*,'C2323=C3131=C1212 = 1/4G'
          print*,'-----------------------------------'
          print*
          go to 72
        ELSE IF(string(1:3).eq.'not')THEN
          print_tensor=.false.
          print*, 'tensor formats disabled.'
          go to 72
        END IF
      END IF
      syntax(iprompt)=.false.
      valid(iprompt)=.false.
      pstring(iprompt)=string
      Lpstring(iprompt)=LNBL77(pstring(iprompt))

      IF(ieq.le.1)GO TO 992
      write(jnkstr,11)ieq-1
      jstrng=pstring(iprompt)(1:ieq-1)
      read(jstrng,jnkstr,err=992)nam(iprompt)
      jstrng=pstring(iprompt)(ieq+1:Lpstring(iprompt))
      read(jstrng,*,err=992)val(iprompt)
      syntax(iprompt)=.true.

        DO i=1,ncnst
          IF(nam(iprompt).eq.NAMcnst(i)
     $    .or.(nam(iprompt).eq.'mu'.and.i.eq.ipG))THEN
             idv(iprompt)=i
             call chkValidity(idv(iprompt),val(iprompt),valid(iprompt))
             if(iprompt.gt.1)then
               IF(idv(iprompt).eq.idv(iprompt-1))THEN
                 if(valid(iprompt))then
                    print*,'---> Replacing FIRST elastic constant by '
     &                    ,      pstring(iprompt)(1:Lpstring(iprompt))
                    val(iprompt-1)=val(iprompt)
                    pstring(iprompt-1)=pstring(iprompt)
                 end if
                 valid(iprompt)=.false.
                 go to 72
               END IF
             endif
             if(valid(iprompt)) iprompt=iprompt+1
             go to 72
          ELSE IF(nam(iprompt).eq.'cs'.and.i.eq.9)THEN
             print*,'      ***********************'
             print*,'      Keyword cs not recognized.'
             print*,'      You probably want cl or ct.'
             print*,'      TRY AGAIN...'
             print*,'      ***********************'
             valid(iprompt)=.false.
             go to 72
          END IF
        END DO
      valid(iprompt)=.false.
      print*,'      ***********************'
      print*,'      UNKNOWN CONSTANT: ',nam(iprompt)
      print*,'        t r y    a g a i n '
      print*,'      ***********************'
      go to 72


   73 continue
      if(verbose)call prna(cnst,NAMcnst,ncnst,'cnst at 73')
      IF(idv(1).ge.ipneedrho.or.idv(2).ge.ipneedrho)THEN
        IF(syntaxRHO.and.validRHO.and.rho.ne.r0)THEN
          print*,'Density? [',stringRHO(1:LstringRHO),']'
        ELSE
          print*,'Density?  (enter non-zero value)'
        END IF
      ELSE
        IF(syntaxRHO.and.validRHO)THEN
           print*,'Density? [',stringRHO(1:LstringRHO),']',
     &            ' (enter zero if wave speeds not desired)'
        ELSE
           print*,'Density?',
     &            ' (enter zero if wave speeds not desired)'
        END IF
      END IF
      if(debug)print*,'about to read string'
      read(*,1)string
      if(debug)print*,'1string=',string
      idum=index(string,bang)
      if (idum.ne.0) string=string(1:idum-1)
      if(debug)print*,'2string=',string
      if(LNBL77(string).eq.0)go to 63
      if(debug)print*,'3string=',string
      syntaxRHO=.false.
      validRHO=.false.
      stringRHO=string
      if(debug)print*,'4string=',string
      LstringRHO=len(stringRHO)
      if(debug)print*,'LstringRHO=',LstringRHO
      call compress(stringRHO,LstringRHO)
      call ChngCase('L',stringRHO)
      JNKchr=stringRHO(1:1)
      IF(   stringRHO.eq.'q' 
     $ .or. stringRHO.eq.'s' 
     $ .or. stringRHO.eq.'e')stop
      IF(JNKchr.eq.'h')call prtHELP(HELPFILE,ierr)
      IF(stringRHO(1:4).eq.'rho=')THEN
        jstrng=stringRHO(5:LstringRHO)
        read(jstrng,*,err=33)rho
      ELSE
      if(debug)print*,'about to read string=',string
        read(stringRHO,*,err=33)rho
      if(debug)print*,'rhovalue=',rho
      END IF
      syntaxRHO=.true.
      IF(rho.ge.r0)THEN
         if(debug)print*,'rho=',rho
         validRHO=.true.
      ELSE
         validRHO=.false.
         print*,'Bad value: density must be positive'
         go to 73
      END IF
      
   63 continue
      if(verbose)call prna(cnst,NAMcnst,ncnst,'cnst at 73')
    

c........................................................... damage begin
c
c     Note to self: enhance this to ask if the specified elastic
c     properties were for the matrix or for the composite.
c     If composite, add code to solve the inverse problem for matrix
c     properties.
c
      fm=r1
      IF(syntaxPORO.and.validPORO)THEN
         print*,'Damage measure?  [',
     &    stringPORO(1:LstringPORO),']'
      ELSE
         print*,'Damage measure? [0]'
         print*,'   Enter zero if damaged properties not desired.'
         print*,'   If positive, damage is void volume fraction.'
         print*,'   If negative, damage is crack radius cubed times'
         print*,'                the number of cracks per volume.'
      END IF
      read(*,1)string
      idum=index(string,bang)
      if (idum.ne.0) string=string(1:idum-1)
      if(LNBL77(string).eq.0)then
         poro=poroDEFAULT
         go to 74
      end if
      syntaxPORO=.false.
      validPORO=.false.
      stringPORO=string
      LstringPORO=len(stringPORO)
      call compress(stringPORO,LstringPORO)
      call ChngCase('L',stringPORO)
      JNKchr=stringPORO(1:1)
      IF(   stringPORO.eq.'q' 
     $ .or. stringPORO.eq.'s' 
     $ .or. stringPORO.eq.'e')stop
      IF(JNKchr.eq.'h')call prtHELP(HELPFILE,ierr)
      IF(stringPORO(1:4).eq.'poro=')THEN
        jstrng=stringPORO(5:LstringPORO)
        read(jstrng,*,err=23)poro
      ELSE
        jstrng=stringPORO(1:LstringPORO)
        read(jstrng,*,err=23)poro
      END IF
      poroDEFAULT=poro
      syntaxPORO=.true.
      IF(poro.ge.r0.and.poro.lt.r1)THEN
         validPORO=.true.
         IF(poro.gt.0.7d0)THEN
           print*,'Warning: large porosity exceeds sensible range'
           print*,'of applicability of elastic constant formulas.'
         END IF
      ELSE IF(poro.lt.r0)THEN
         validPORO=.true.
         print*,'OKAY -Poro interpreted as crack parameter,'
         print*,'number per volume times crack radius cubed.'
      ELSE
         validPORO=.false.
         print*,'Bad value: porosity must be less than one'
         go to 63
      END IF
      
      if(validPORO.and.syntaxPORO)damage=.true.

   74 continue
      if(poro.eq.r0)damage=.false.
c........................................................... damage end
      
      J1=idv(1)
      J2=idv(2)
      cnst(J1)=val(1)
      cnst(J2)=val(2)

      IF (J1.eq.ipko)THEN
         J1=ipnu
         cnst(ipnu)=val(1)/(r1+val(1))
      ELSE IF(J1.eq.ipcl)THEN
         J1=ipH
         cnst(ipH)=rho*val(1)*val(1)
      ELSE IF(J1.eq.ipct)THEN
         J1=ipG
         cnst(ipG)=rho*val(1)*val(1)
      ELSE IF(J1.eq.ipco)THEN
         J1=ipK
         cnst(ipK)=rho*val(1)*val(1)
      ELSE IF(J1.eq.ipcr)THEN
         J1=ipE
         cnst(ipE)=rho*val(1)*val(1)
      END IF

      IF (J2.eq.ipko)THEN
         J2=ipnu
         cnst(ipnu)=val(2)/(r1+val(2))
      ELSE IF(J2.eq.ipcl)THEN
         J2=ipH
         cnst(ipH)=rho*val(2)*val(2)
      ELSE IF(J2.eq.ipct)THEN
         J2=ipG
         cnst(ipG)=rho*val(2)*val(2)
      ELSE IF(J2.eq.ipco)THEN
         J2=ipK
         cnst(ipK)=rho*val(2)*val(2)
      ELSE IF(J2.eq.ipcr)THEN
         J2=ipE
         cnst(ipE)=rho*val(2)*val(2)
      END IF

      IF(rho.le.r0.and.(idv(1).ge.ipneedrho.or.idv(2).ge.ipneedrho))THEN
         print*,'Density must be positive when a wave speed is given'
         validRHO=.false.
         go to 73
      END IF


      IF(J1.lt.J2)THEN
         icase=10*J1+J2
      ELSE IF(J1.gt.J2)THEN
         icase=10*J2+J1
      ELSE
        print*,'           ___________________________'
        print*,'          | constants not independent |'
        print*,'          |_________START OVER________|'
        go to 71
      END IF

         lam = cnst(iplam)
           G = cnst(ipG)
           E = cnst(ipE)
          nu = cnst(ipnu)
           K = cnst(ipK)
           H = cnst(ipH)
       if(verbose)then
           print*,NAMcnst(J1),'=',cnst(J1)
           print*,NAMcnst(J2),'=',cnst(J2)
           print*,'icase=',icase
       endif
           

c At this point, J1 and J2 each range from 1 to 6, and are distinct.
c There are 15 possible ways to choose 2 numbers from 6:

c     Get G and nu
      IF(icase.eq.12)THEN                                       ! lam, G
         if(lam+G.eq.r0)go to 91
         nu = lam/r2/(lam+G)
      ELSE IF(icase.eq.13)THEN                                  ! lam, E
         A  = E*E + r2*lam*E + r9*lam*lam
         IF(A.lt.r0)go to 91
         A = sqrt(A)
         G  = (A - r3*lam+E)/r4
         nu = (A-E-lam)/r4/lam
      ELSE IF(icase.eq.14)THEN                                  ! lam,nu
         RNU=nu
         if(RNU.eq.r0)RNU=puny
         G  = lam*(r1-r2*RNU)/r2/RNU
      ELSE IF(icase.eq.15)THEN                                  ! lam, K
         if(r3*K-lam.eq.r0)go to 91
         G  = r3*(K-lam)/r2
         nu = lam/(r3*K-lam)
      ELSE IF(icase.eq.16)THEN                                  ! lam, H
         if(H+lam.eq.r0)go to 91
         G  = (H-lam)/r2
         nu = lam/(H+lam)


      ELSE IF(icase.eq.23)THEN                                  !   G, E
         nu = (E-r2*G)/r2/G
      ELSE IF(icase.eq.24)THEN                                  !   G,nu
          G = G
         nu = nu
      ELSE IF(icase.eq.25)THEN                                  !   G, K
         if(r3*K+G.eq.r0)go to 91
         nu = (r3*K-r2*G)/r2/(r3*K+G)
      ELSE IF(icase.eq.26)THEN                                  !   G, H
         if(H-G.eq.r0)go to 91
         nu = (H-r2*G)/r2/(H-G)
      ELSE IF(icase.eq.34)THEN                                  !   E,nu
         if(r1+nu.eq.r0)go to 91
         G  = E/r2/(r1+nu)
      ELSE IF(icase.eq.35)THEN                                  !   E, K
         if(r9*K-E.eq.r0 .or. K.eq.r0)go to 91
         G  = r3*E*K/(r9*K-E)
         nu = (r3*K-E)/r6/K
      ELSE IF(icase.eq.36)THEN                                  !   E, H
         B = E*E+r9*H*H-r10*E*H
         IF(B.lt.r0 .or. H.eq.r0)go to 91
         B = sqrt(B)
         G  = (r3*H-B+E)/r8
         nu = (B-H+E)/r4/H


      ELSE IF(icase.eq.45)THEN                                  !  nu, K
         if(r1+nu.eq.r0)go to 91
         G  = r3*K*(r1-r2*nu)/r2/(r1+nu)
      ELSE IF(icase.eq.46)THEN                                  !  nu, H
         if(r1-nu.eq.r0)go to 91
         G  = H*(r1-r2*nu)/r2/(r1-nu)


      ELSE IF(icase.eq.56)THEN                                  !   K, H
         if(r3*K+H.eq.r0)go to 91
         G  = r3*(H-K)/r4
         nu = (r3*K-H)/(r3*K+H)


      END IF

  111 RNU=nu
      if(RNU.eq.r0)RNU=puny
      
      om2n = r1-r2*RNU
      omn  = r1-RNU
      opn  = r1+RNU
      r2g = r2*G

      lam = r2g*RNU/om2n
      E   = r2g*opn
      K   = r2g*opn/r3/om2n
      H   = r2g*omn/om2n
      ko  = nu/omn

      if(G.le.r0  .or.  K.le.r0)then
         print*,'    ~~BAD~BAD~BAD~BAD~BAD~BAD~BAD~BAD~~'
         print*,'    | BAD   lam = ',lam
         print*,'    | BAD     G = ',G
         print*,'    | BAD     E = ',E
         print*,'    | BAD    nu = ',nu
         print*,'    | BAD     K = ',K
         print*,'    | BAD     H = ',H
         go to 91
      end if

      cnst(1) = lam
      cnst(2) = G
      cnst(3) = E
      cnst(4) = nu
      cnst(5) = K
      cnst(6) = H
      cnst(7) = ko
      
      IF(rho.ne.r0)THEN
        ilast = ncnst
        cnst(8) = sqrt(H/rho/fm)
        cnst(9) = sqrt(G/rho/fm)
        cnst(10) = sqrt(K/rho/fm)
        cnst(11) = sqrt(E/rho/fm)
      else
        ilast = 7
      END IF

      if(damage)then
         print*
         print*
         print*,'MATRIX MATERIAL PROPERTIES'
      end if
      print*,'            ~~~~~~~~~~~~~~~~~~~~~~'
      do i=1,ilast
        if(i.eq.4.or.i.eq.7)then
          print221,NAMcnst(i), cnst(i)
        else
          print222,NAMcnst(i), cnst(i)
        end if
      END DO
      if(rho.ne.r0)then
             print222,'rho',rho*fm
             print*,'rho*cl',rho*cnst(8)
             print*,'rho*ct',rho*cnst(9)
      endif
      print*,'            ~~~~~~~~~~~~~~~~~~~~~~'
      print223,
     &'uniaxialYIELD/sigHEL = 2G/H = (1-2nu)/(1-nu)',r1-ko
      if(single_line_output)then
        print*,'###################################################'
        if(rho.gt.r0)then
        write(jnkstr,2)(NAMcnst(i),cnst(i), i=1,ilast),'rho',rho
        else
        write(jnkstr,2)(NAMcnst(i),cnst(i), i=1,ilast)
        endif
        idum=len(jnkstr)
c///////call compress(jnkstr,len(jnkstr)) ! causes segfault on linux
        call compress(jnkstr,idum)
        idum=LNBL77(jnkstr)
        jnkstr(idum:idum)='}'
        print*,jnkstr(1:idum)
c        print2,(NAMcnst(i),cnst(i), i=1,ilast)
        print*,'###################################################'
        print*
      end if
      if(print_tensor)then
         print*
         print*
         print*,'############# TENSOR COMPONENTS ###########'
         print*
         print3,'E11=E22=E33 =   H = ',cnst(6)
         print3,'E12=E23=E31 = lam = ',cnst(1)
         print3,'E44=E55=E66 =  2G = ',r2*cnst(2)
         print*,'all other Eij=0'
         print*
         print3,'C11=C22=C33 =   1/E  = ',r1/cnst(3)
         print3,'C12=C13=C23 = -nu/E  = ',-cnst(4)/cnst(3)
         print3,'C44=C55=C66 =  1/2G  = ',r1/r2/cnst(2)
         print*,'all other Cij=0'
         print*
      end if
      print*
      print*
      if(damage)then
       IF(poro.gt.r0)THEN
         fm=r1-poro
         alpha=(r1+nu)/(r1-nu)/r3
         K=K*fm*(r1-alpha)/(r1-fm*alpha)
         beta=r2*(r4-r5*nu)/(r1-nu)/r15
         G=G*fm*(r1-beta)/(r1-fm*beta)
         nu = (r3*K-r2*G)/r2/(r3*K+G)
         print*
         print*,'POROUS COMPOSITE ELASTIC PROPERTIES'
         IF(poro.gt.0.7d0)THEN
           print*,'Warning: large porosity exceeds sensible range'
           print*,'of applicability of elastic constant formulas.'
         END IF
         poro=r0
         go to 111
       ELSE IF(poro.lt.r0)then
         fm=r1
         poro=-poro
         G=G/(r1+poro*r32*(r5-nu)*(r1-nu)/r45/(r2-nu))
         K=K/(r1+poro*r16*(r1-nu*nu)/r9/(1-r2*nu))
         nu = (r3*K-r2*G)/r2/(r3*K+G)
         poro=r0
         print*
         print*,'CRACKED COMPOSITE ELASTIC PROPERTIES'
         go to 111
       END IF
      endif
      print*
      print*,'Ready for a new pair of constants...'
      print*
      print*,'___________ TYPE q TO QUIT ___________'
      go to 71

   33 print*,'      ****** ERROR READING DENSITY ******'
      print*,'              t r y    a g a i n '
      print*,'      ***********************************'
      go to 73
   23 print*,'      ******* ERROR READING VALUE *******'
      print*,'              t r y    a g a i n '
      print*,'      ***********************************'
      go to 63
   91 if(verbose)then
         call prna(cnst,NAMcnst,ncnst,'cnst at 73')
      end if
      print*,'                                           '
      print*,'                                           '
      print*,'                                           '
      print*,'    !~~~~~~~~~ INVALID CONSTANTS ~~~~~~~~~!'
      print*,'    !      non-positive definiteness      !'
      print*,'    !~~~~~~~~~~~~~START OVER~~~~~~~~~~~~~~!'
      print*,'                                           '
      go to 71
  992 call bad_syntax
      syntax(iprompt)=.false.
      go to 72
C---.----1----.----2----.----3----.----4----.----5----.----6----.----7--
    1 format(50a)
    2 format(1x,'elas:{',A3,'=',g16.7,19(', ',A3,'=',g16.7))
C   3 format(A,3(t20,g16.7,/))
    3 format(A,t22,g15.7)
    4 format(a)
   11 format('(',i2,'a)')
  221 format(13x,A3,' =',F13.7)
  222 format(13x,A3,' =',G16.7)
  223 format(13x,A,' = ',F13.7)
      end

c---.----1----.----2----.----3----.----4----.----5----.----6----.----7--

      subroutine compress (string,last)
c        This routine removes all blanks in a string from
c        character 1 to last, and shifts the remainder of the
c        string to the left.
c
c     UPON INPUT
c     ----------
c     ~string.......a character string
c     ~last.........index of last character in the
c                   "to be compressed" part of the string
c     UPON RETURN
c     -----------
c     ~string.......string with all blanks in the compression interval
c                   removed and all subsequent characters shifted to
c                   the left by an amount equal to the number of blanks
c                   removed.
c     ~last.........new index of the last character in the compressed
c                   part of the string
c
      implicit none
      character*(*) string
      character*1 BL
      parameter (BL=' ')
      integer i,j,knt,nrb,length,last,   LNBL77
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      length = LNBL77(string)
      last = min(last,length)

      knt=0
      do i=1,last
         IF(string(i:i).ne.BL) then
            knt = knt+1
            string(knt:knt)=string(i:i)
         END IF
      END DO

      nrb = last-knt  ! number of removed blanks

      do i=last+1,length
         j=i-nrb
         string(j:j)=string(i:i)
      END DO

      j=length-nrb  ! this line has to be here in case last+1>length

      do i=j+1,length
         string(i:i)=BL
      END DO

      last = last-nrb

      RETURN
      end
c---.----1----.----2----.----3----.----4----.----5----.----6----.----7--

c     FUNCTION LNBL77(string)
c     ---- Sparc FORTRAN intrinsic function ---
c     LNBL77 = length of string with trailing blanks removed

c---.----1----.----2----.----3----.----4----.----5----.----6----.----7--

c     FUNCTION index(string1,string2)
c     ---- Sparc FORTRAN intrinsic function ---
c     index = index of the first occurance of string1 in string2
c           = 0 if string1 not present

c---.----1----.----2----.----3----.----4----.----5----.----6----.----7--
      subroutine ChngCase(option,str)
      implicit none
      integer aLpHa,ZeTa,shift,I,LNBL77,idum
      character*1 tic,cdum
      parameter(tic='''')
      character*(*) str, option
      logical LaA,Aa,quotes
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      LaA = (option.eq.'aA' .or. option.eq.'U'. or. option.eq.'u')
      Aa = (option.eq.'Aa'.or.option.eq.'L'.or.option.eq.'l')
      IF (Aa) then
        aLpHa=ICHAR('A')
        shift=ICHAR('a')-aLpHa
      ELSE IF (LaA) then
        aLpHa=ICHAR('a')
        shift=ICHAR('A')-aLpHa
      else
        RETURN
      END IF
      ZeTa=aLpHa+25

      quotes=.false.
      do I=1,LNBL77(str)
        cdum=str(I:I)
        idum=ICHAR(cdum)
        IF(cdum.eq.tic)quotes=.not.quotes
        IF(             ! character is one that gets changed.
     |  (idum.ge.aLpHa .and. idum.le.ZeTa)
     |  .AND.           ! character is not in a quoted sub-string.
     |  .not.quotes
     |     )  str(I:I) = char(idum+shift)
      END DO
      RETURN
      END
c---.----1----.----2----.----3----.----4----.----5----.----6----.----7--
      subroutine bad_syntax
      print*,'      **************************************'
      print*,'      SYNTAX SHOULD BE: VARIABLEname = value'
      print*,'                t r y    a g a i n '
      print*,'      **************************************'
      return
      end
c---.----1----.----2----.----3----.----4----.----5----.----6----.----7--
      subroutine chkValidity(I,val,valid)
!
! Positive definiteness requires that
!
!    I      restriction
!   ---     -----------
!    2      G > 0
!    3      E > 0
!    5      K > 0
!    6      H > 0
!    4      -1 < nu < .5
!    7      -.5 < ko < 1
!    8      cl > 0
!    9      ct > 0
!   10      co > 0
!   11      cr > 0
!
! The constant lam is also restricted to have the same sign as nu, but
! the value of nu may not have been given by the time this routine is 
! called, so bad lam values will be caught later on.  This routine 
! merely gets the blatent errors.
!
! Because the validity of a constant is not determined hands down,
! the constant will be considered "innocent until proven guilty".
! That is, this routine will return valid=.true. unless the constant
! can be proved bad beyond the shadow of a doubt.  For this routine,
! positive SEMI-definiteness will be permitted.
!
      implicit none
      integer I
      real*8
     &        val, r0, r1, phalf
      parameter(r0=0.0d0,r1=0.1d1,phalf=0.5d0)
      integer iplam,ipG,ipE,ipnu,ipK,ipH,ipko,ipcl,ipct,ipco,ipcr
      integer ipneedrho,ipLAST
      parameter (iplam=1)
      parameter (ipG  =2)
      parameter (ipE  =3)
      parameter (ipnu =4)
      parameter (ipK  =5)
      parameter (ipH  =6)
      parameter (ipko =7)
      parameter (ipneedrho=8)
      parameter (ipcl =8)
      parameter (ipct =9)
      parameter (ipco =10)
      parameter (ipcr =11)
      parameter (ipLAST =11)
      logical valid
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      valid = .true.
      if(I.eq.iplam)return
      
      if(I.eq.ipnu)then
         if(val.lt.-r1)then
            print9,'< -1'
            valid=.false.
         else if(val.gt.phalf)then
            print9,'> 1/2'
            valid=.false.
         else if(val.eq.-r1)then
            print*,'#########################'
            print*,'changing to nu=-0.9999999'
            print*,'#########################'
            val=-0.9999999d0
         else if(val.eq.phalf)then
            print*,'###########################################'
            print*,'Nearly incompressible or nearly shear free,'
            print*,'depending on your second elastic constant.'
            print*,'Changing to nu= 0.4999999 (incompressible)'
            print*,'###########################################'
            val= 0.4999999d0
         end if
         return
      end if
      
      if(I.eq.ipko)then
         if(val.lt.-phalf)then
            print9,'< -1/2'
            valid=.false.
         else if(val.gt.r1)then
            print9,'> 1'
            valid=.false.
         end if
         return
      end if

      
      if(I.gt.ipLAST)then
         print*,'ERROR: input I to chkValidity unknown'
         valid=.false.
      end if
c
c     All other values for I merely need val>0
      if(val.lt.r0)then
         print9,'< 0'
         valid=.false.
      end if
      return
    9 format(   T5,40('!'),/,
     &          T5,'Bad input: value can''t be ',A,/
     &          T5,'t r y     a g a i n',/,
     &          T5,40('!')   )
      end
      
c---.----1----.----2----.----3----.----4----.----5----.----6----.----7--
      subroutine prtHELP(helpfile,ierr)
      implicit none
      character*1 bang,chr
      integer ierr,lstr
      integer LNBL77
      logical dejavu
      parameter (bang = '!')
      character*(*) helpfile
      character JNKstr*80
      data dejavu/.false./
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      ierr=-999
        open (33,file=helpfile,status='old',err=98)
c 
c First scroll down to the first line that starts with a bang.
C ___________________________________________________(INFINITE LOOP)___
   70   read (33,'(1a)',end=75) chr!                                   \.
        call ChngCase('L',chr)                                         !
        if(chr.ne.bang) go to 70                                       !
c _____________________________________________________________________/
C ___________________________________________________(INFINITE LOOP)___
   71   read (33,'(80a)',end=75) JNKstr!                               \.
            if(JNKstr(1:1).ne.bang) then                               !
               go to 75                                                !
            else                                                       !
               lstr=LNBL77(JNKstr)                                     !
               print*,JNKstr(2:lstr)                                   !
            endif                                                      !
        GO TO 71!______________________________________________________/
   75 close(33)
      ierr=0
      return
   98 continue
      ierr=8
      if(.not.dejavu)then
      dejavu=.true.
      print*
      print*,'Given any two INDEPENDENT elastic constants (from the list
     $below)'
      print*,'this program will compute all of the others.'
      print*
      print*,'    Specify your known input like this:'
      print*
      print*,'       FIRST elastic constant?'
      print*,'      E=200e9                               <--- MATRIX VA
     $LUE IF CRACKED/POROUS'
      print*,'       SECOND elastic constant?'
      print*,'      G=79e9                                <--- MATRIX VA
     $LUE IF CRACKED/POROUS'
      print*,'     Density? [0] (enter zero if wave speeds not desired)'
      print*,'      7860'
      print*,'       Damage measure? [0]'
      print*,'      .04'
      print*
      print*,'Defaults [in brackets] may be accepted by hitting return.'
      print*,'This program runs as an infinite loop, allowing you to ent
     $er multiple input'
      print*,'sets. Your OLD inputs become the NEW defaults for subseque
     $nt loops (which is'
      print*,'useful for correcting typos). Type q, s, or e when you are
     $ done.'
      end if
      print*
      print*,'  ########################################################
     $#####'
      print*,'  lam = First Lame parameter         = G(E-2G)/(3G-E) = K-
     $2G/3'
      print*,'    G = Shear modulus (= 2nd Lame parameter, mu)   = E/2/(
     $1+nu)'
      print*,'    E = Young''s modulus              = 3K(1-2nu) = 2G(1+n
     $u) = 9KG/(3K+G)'
      print*,'   nu = Poisson''s ratio              = (3K-E)/6K = lam/2/
     $(lam+G)'
      print*,'    K = bulk modulus                 = E/3/(1-2nu) = lam +
     $ 2G/3'
      print*,'    H = constrained modulus          = 2G+lam = 3K-2lam = 
     $K + 4G/3'
      print*,'   ko = SIGy/SIGx in uniaxial strain = nu/(1-nu)'
      print*,'   cl = longitudinal wave speed      = sqrt(H/rho)'
      print*,'   ct = shear (TRANSVERSE) wave speed  = sqrt(G/rho)'
      print*,'   co = bulk/plastic wave speed      = sqrt(K/rho)=SQRT(cl
     $^2-4(ct^2)/3)'
      print*,'   cr = thin rod elastic wave speed  = sqrt(E/rho)'
      print*,'  ########################################################
     $#####'
      print*,'  NOTE: when shock physicists say `cs'', they mean what we
     $ call `cl''.'
      print*
      print*,'To see this list again, type "help".'
      print*,'To stop, type  q, s, or e.'
      return
      end
c---.----1----.----2----.----3----.----4----.----5----.----6----.----7--
      subroutine prna(arr,anam,idim,msg)
c     print out a named array
      implicit none
      integer idim,i
      character*(*) msg
      character*(*) anam
      dimension anam(*)
      real*8  arr(idim)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      print*,msg
      do i=1,idim
         print*,i,':  ',anam(i),'=',arr(i)
      end do
      return
      end
c---.----1----.----2----.----3----.----4----.----5----.----6----.----7--
      subroutine pra(arr,idim,msg)
c     print out an array (also see subroutine prna)
      implicit none
      integer idim,i
      character*(*) msg
      real*8
     &        arr(idim)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      print*,msg
      do i=1,idim
         print*,i,':  ',arr(i)
      end do
      return
      end
c---.----1----.----2----.----3----.----4----.----5----.----6----.----7--
      FUNCTION LNBL77(STR)
c     This function returns the index of the last non-blank character
c     in STR, or zero if the string is all blanks.
c  >>>In FORTRAN 90, this is an intrinsic function called LNBLNK.
      IMPLICIT NONE
      CHARACTER*(*) STR
      CHARACTER*1 BLANK
      PARAMETER (BLANK=' ')
      INTEGER LNBL77,I,IDUM
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      IDUM=LEN(STR)
      DO 100 I=IDUM,1,-1
         IF(STR(I:I).NE.BLANK) GO TO 777
  100 CONTINUE
  777 LNBL77 = I
      RETURN
      END

